@@ -10,6 +10,12 @@ fi
 # install deps
 ./contrib/ci/fwupd_setup_helpers.py --yes -o fedora -v mingw64 install-dependencies
 
+# until it appears in updates-testing
+dnf install --allowerasing -y \
+    https://kojipkgs.fedoraproject.org//packages/msitools/0.102/1.fc39/x86_64/msitools-0.102-1.fc39.x86_64.rpm \
+    https://kojipkgs.fedoraproject.org//packages/glib2/2.76.4/3.fc38/x86_64/glib2-2.76.4-3.fc38.x86_64.rpm \
+    https://kojipkgs.fedoraproject.org//packages/glib2/2.76.4/3.fc38/x86_64/glib2-devel-2.76.4-3.fc38.x86_64.rpm
+
 # update to latest version of meson
 if [ "$(id -u)" -eq 0 ]; then
     dnf install -y python-pip
@@ -47,7 +53,6 @@ meson setup .. \
     -Dfirmware-packager=false \
     -Dmetainfo=false \
     -Dcompat_cli=false \
-    -Dsoup_session_compat=false \
     -Dgcab:introspection=false \
     -Dgcab:docs=false \
     -Dgcab:nls=false \
@@ -76,7 +81,7 @@ ninja -C "$build" -v install
 
 #disable motd for Windows
 cd $root
-sed -i 's,UpdateMotd=.*,UpdateMotd=false,' "$DESTDIR/etc/fwupd/daemon.conf"
+sed -i 's,UpdateMotd=.*,UpdateMotd=false,' "$DESTDIR/etc/fwupd/fwupd.conf"
 
 # create a setup binary
 CERTDIR=/etc/pki/tls/certs
