@@ -27,11 +27,14 @@
  * See also: [class@FwupdClient]
  */
 
+#define FWUPD_REMOTE_CONFIG_DEFAULT_REFRESH_INTERVAL 86400 /* 24h */
+
 static void
 fwupd_remote_finalize(GObject *obj);
 
 typedef struct {
 	FwupdRemoteKind kind;
+	FwupdRemoteFlags flags;
 	FwupdKeyringKind keyring_kind;
 	gchar *id;
 	gchar *firmware_base_uri;
@@ -43,19 +46,17 @@ typedef struct {
 	gchar *password;
 	gchar *title;
 	gchar *agreement;
-	gchar *checksum;
+	gchar *checksum;     /* of metadata */
+	gchar *checksum_sig; /* of the signature */
 	gchar *filename_cache;
 	gchar *filename_cache_sig;
 	gchar *filename_source;
-	gboolean enabled;
-	gboolean approval_required;
 	gint priority;
 	guint64 mtime;
+	guint64 refresh_interval;
 	gchar **order_after;
 	gchar **order_before;
 	gchar *remotes_dir;
-	gboolean automatic_reports;
-	gboolean automatic_security_reports;
 } FwupdRemotePrivate;
 
 enum {
@@ -65,18 +66,69 @@ enum {
 	PROP_APPROVAL_REQUIRED,
 	PROP_AUTOMATIC_REPORTS,
 	PROP_AUTOMATIC_SECURITY_REPORTS,
+	PROP_FLAGS,
 	PROP_LAST
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE(FwupdRemote, fwupd_remote, G_TYPE_OBJECT)
 #define GET_PRIVATE(o) (fwupd_remote_get_instance_private(o))
 
-#ifdef HAVE_LIBCURL_7_62_0
+#ifdef HAVE_LIBCURL
 typedef gchar curlptr;
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(curlptr, curl_free)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(CURLU, curl_url_cleanup)
 #endif
 
+/**
+ * fwupd_remote_flag_to_string:
+ * @flag: remote attribute flags, e.g. %FWUPD_REMOTE_FLAG_ENABLED
+ *
+ * Returns the printable string for the flag.
+ *
+ * Returns: string, or %NULL
+ *
+ * Since: 1.9.4
+ **/
+const gchar *
+fwupd_remote_flag_to_string(FwupdRemoteFlags flag)
+{
+	if (flag == FWUPD_REMOTE_FLAG_NONE)
+		return "none";
+	if (flag == FWUPD_REMOTE_FLAG_ENABLED)
+		return "enabled";
+	if (flag == FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED)
+		return "approval-required";
+	if (flag == FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS)
+		return "automatic-reports";
+	if (flag == FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS)
+		return "automatic-security-reports";
+	return NULL;
+}
+
+/**
+ * fwupd_remote_flag_from_string:
+ * @flag: (nullable): a string, e.g. `enabled`
+ *
+ * Converts a string to an enumerated flag.
+ *
+ * Returns: enumerated value
+ *
+ * Since: 1.9.4
+ **/
+FwupdRemoteFlags
+fwupd_remote_flag_from_string(const gchar *flag)
+{
+	if (g_strcmp0(flag, "enabled") == 0)
+		return FWUPD_REMOTE_FLAG_ENABLED;
+	if (g_strcmp0(flag, "approval-required") == 0)
+		return FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED;
+	if (g_strcmp0(flag, "automatic-reports") == 0)
+		return FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS;
+	if (g_strcmp0(flag, "automatic-security-reports") == 0)
+		return FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS;
+	return FWUPD_REMOTE_FLAG_NONE;
+}
+
 /**
  * fwupd_remote_to_json:
  * @self: a #FwupdRemote
@@ -115,22 +167,127 @@ fwupd_remote_to_json(FwupdRemote *self, JsonBuilder *builder)
 	fwupd_common_json_add_string(builder, "Title", priv->title);
 	fwupd_common_json_add_string(builder, "Agreement", priv->agreement);
 	fwupd_common_json_add_string(builder, "Checksum", priv->checksum);
+	fwupd_common_json_add_string(builder, "ChecksumSig", priv->checksum_sig);
 	fwupd_common_json_add_string(builder, "FilenameCache", priv->filename_cache);
 	fwupd_common_json_add_string(builder, "FilenameCacheSig", priv->filename_cache_sig);
 	fwupd_common_json_add_string(builder, "FilenameSource", priv->filename_source);
-	fwupd_common_json_add_boolean(builder, "Enabled", priv->enabled);
-	fwupd_common_json_add_boolean(builder, "ApprovalRequired", priv->approval_required);
-	fwupd_common_json_add_boolean(builder, "AutomaticReports", priv->automatic_reports);
+	fwupd_common_json_add_int(builder, "Flags", priv->flags);
 	fwupd_common_json_add_boolean(builder,
-				      "AutomaticSecurityReports",
-				      priv->automatic_security_reports);
+				      "Enabled",
+				      fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_ENABLED));
+	fwupd_common_json_add_boolean(
+	    builder,
+	    "ApprovalRequired",
+	    fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED));
+	fwupd_common_json_add_boolean(
+	    builder,
+	    "AutomaticReports",
+	    fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS));
+	fwupd_common_json_add_boolean(
+	    builder,
+	    "AutomaticSecurityReports",
+	    fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS));
 	fwupd_common_json_add_int(builder, "Priority", priv->priority);
 	fwupd_common_json_add_int(builder, "Mtime", priv->mtime);
+	fwupd_common_json_add_int(builder, "RefreshInterval", priv->refresh_interval);
 	fwupd_common_json_add_string(builder, "RemotesDir", priv->remotes_dir);
 	fwupd_common_json_add_stringv(builder, "OrderAfter", priv->order_after);
 	fwupd_common_json_add_stringv(builder, "OrderBefore", priv->order_before);
 }
 
+/**
+ * fwupd_remote_get_flags:
+ * @self: a #FwupdRemote
+ *
+ * Gets the self flags.
+ *
+ * Returns: remote attribute flags, or 0 if unset
+ *
+ * Since: 1.9.4
+ **/
+FwupdRemoteFlags
+fwupd_remote_get_flags(FwupdRemote *self)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), 0);
+	return priv->flags;
+}
+
+/**
+ * fwupd_remote_set_flags:
+ * @self: a #FwupdRemote
+ * @flags: remote attribute flags, e.g. %FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED
+ *
+ * Sets the attribute flags.
+ *
+ * Since: 1.9.4
+ **/
+void
+fwupd_remote_set_flags(FwupdRemote *self, FwupdRemoteFlags flags)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+	if (flags == priv->flags)
+		return;
+	priv->flags = flags;
+	g_object_notify(G_OBJECT(self), "flags");
+}
+
+/**
+ * fwupd_remote_add_flag:
+ * @self: a #FwupdRemote
+ * @flag: the #FwupdRemoteFlags, e.g. %FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED
+ *
+ * Adds a specific attribute flag to the attribute.
+ *
+ * Since: 1.9.4
+ **/
+void
+fwupd_remote_add_flag(FwupdRemote *self, FwupdRemoteFlags flag)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+	priv->flags |= flag;
+	g_object_notify(G_OBJECT(self), "flags");
+}
+
+/**
+ * fwupd_remote_remove_flag:
+ * @self: a #FwupdRemote
+ * @flag: the #FwupdRemoteFlags, e.g. %FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED
+ *
+ * Removes a specific attribute flag from the remote.
+ *
+ * Since: 1.9.4
+ **/
+void
+fwupd_remote_remove_flag(FwupdRemote *self, FwupdRemoteFlags flag)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+	priv->flags &= ~flag;
+	g_object_notify(G_OBJECT(self), "flags");
+}
+
+/**
+ * fwupd_remote_has_flag:
+ * @self: a #FwupdRemote
+ * @flag: the remote flag, e.g. %FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED
+ *
+ * Finds if the remote has a specific flag.
+ *
+ * Returns: %TRUE if the flag is set
+ *
+ * Since: 1.9.4
+ **/
+gboolean
+fwupd_remote_has_flag(FwupdRemote *self, FwupdRemoteFlags flag)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
+	return (priv->flags & flag) > 0;
+}
+
 static gchar *
 fwupd_strdup_nonempty(const gchar *text)
 {
@@ -152,11 +309,22 @@ fwupd_remote_set_username(FwupdRemote *self, const gchar *username)
 	priv->username = g_strdup(username);
 }
 
-static void
+/**
+ * fwupd_remote_set_title:
+ * @self: a #FwupdRemote
+ * @title: (nullable): title text, e.g. "Backup"
+ *
+ * Sets the remote title.
+ *
+ * Since: 1.8.13
+ **/
+void
 fwupd_remote_set_title(FwupdRemote *self, const gchar *title)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+
 	/* not changed */
 	if (g_strcmp0(priv->title, title) == 0)
 		return;
@@ -179,6 +347,8 @@ fwupd_remote_set_agreement(FwupdRemote *self, const gchar *agreement)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+
 	/* not changed */
 	if (g_strcmp0(priv->agreement, agreement) == 0)
 		return;
@@ -190,17 +360,36 @@ fwupd_remote_set_agreement(FwupdRemote *self, const gchar *agreement)
 /**
  * fwupd_remote_set_checksum:
  * @self: a #FwupdRemote
- * @checksum: (nullable): checksum string
+ * @checksum_sig: (nullable): checksum string
  *
- * Sets the remote checksum, typically only useful in the self tests.
+ * Sets the remote signature checksum, typically only useful in the self tests.
+ *
+ * NOTE: This should have been called fwupd_remote_set_checksum_sig() but alas, ABI.
  *
  * Since: 1.8.2
  **/
 void
-fwupd_remote_set_checksum(FwupdRemote *self, const gchar *checksum)
+fwupd_remote_set_checksum(FwupdRemote *self, const gchar *checksum_sig)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+
+	/* not changed */
+	if (g_strcmp0(priv->checksum_sig, checksum_sig) == 0)
+		return;
+
+	g_free(priv->checksum_sig);
+	priv->checksum_sig = g_strdup(checksum_sig);
+}
+
+static void
+fwupd_remote_set_checksum_metadata(FwupdRemote *self, const gchar *checksum)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+
 	/* not changed */
 	if (g_strcmp0(priv->checksum, checksum) == 0)
 		return;
@@ -242,11 +431,22 @@ void
 fwupd_remote_set_keyring_kind(FwupdRemote *self, FwupdKeyringKind keyring_kind)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
 	priv->keyring_kind = keyring_kind;
 }
 
-/* note, this has to be set before url */
-static void
+/**
+ * fwupd_remote_set_id:
+ * @self: a #FwupdRemote
+ * @id: (nullable): remote ID, e.g. "lvfs"
+ *
+ * Sets the remote title.
+ *
+ * NOTE: the ID has to be set before the URL.
+ *
+ * Since: 1.9.3
+ **/
+void
 fwupd_remote_set_id(FwupdRemote *self, const gchar *id)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
@@ -273,6 +473,7 @@ void
 fwupd_remote_set_filename_source(FwupdRemote *self, const gchar *filename_source)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
 	if (priv->filename_source == filename_source)
 		return;
 	g_free(priv->filename_source);
@@ -292,15 +493,27 @@ fwupd_remote_get_suffix_for_keyring_kind(FwupdKeyringKind keyring_kind)
 }
 
 static gchar *
-fwupd_remote_build_uri(FwupdRemote *self, const gchar *url, GError **error)
+fwupd_remote_build_uri(FwupdRemote *self,
+		       const gchar *base_uri,
+		       const gchar *url_noauth,
+		       GError **error)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
-#ifdef HAVE_LIBCURL_7_62_0
+#ifdef HAVE_LIBCURL
+	g_autofree gchar *url = NULL;
 	g_autoptr(curlptr) tmp_uri = NULL;
 	g_autoptr(CURLU) uri = curl_url();
 
+	/* the LVFS can't accept basic auth on an endpoint not expecting authentication */
+	if (!g_str_has_suffix(url_noauth, "/auth") &&
+	    (priv->username != NULL || priv->password != NULL)) {
+		url = g_strdup_printf("%s/auth", url_noauth);
+	} else {
+		url = g_strdup(url_noauth);
+	}
+
 	/* create URI, substituting if required */
-	if (priv->firmware_base_uri != NULL) {
+	if (base_uri != NULL) {
 		g_autofree gchar *basename = NULL;
 		g_autofree gchar *path_new = NULL;
 		g_autoptr(curlptr) path = NULL;
@@ -348,33 +561,49 @@ fwupd_remote_build_uri(FwupdRemote *self, const gchar *url, GError **error)
 		}
 	}
 
-	/* set the username and password */
-	if (priv->username != NULL)
-		(void)curl_url_set(uri, CURLUPART_USER, priv->username, 0);
-	if (priv->password != NULL)
-		(void)curl_url_set(uri, CURLUPART_PASSWORD, priv->password, 0);
+	/* set the escaped username and password */
+	if (priv->username != NULL) {
+		g_autofree gchar *user_escaped = g_uri_escape_string(priv->username, NULL, FALSE);
+		(void)curl_url_set(uri, CURLUPART_USER, user_escaped, 0);
+	}
+	if (priv->password != NULL) {
+		g_autofree gchar *pass_escaped = g_uri_escape_string(priv->password, NULL, FALSE);
+		(void)curl_url_set(uri, CURLUPART_PASSWORD, pass_escaped, 0);
+	}
 	(void)curl_url_get(uri, CURLUPART_URL, &tmp_uri, 0);
 	return g_strdup(tmp_uri);
 #else
 	if (priv->firmware_base_uri != NULL) {
-		g_autofree gchar *basename = g_path_get_basename(url);
+		g_autofree gchar *basename = g_path_get_basename(url_noauth);
 		return g_build_filename(priv->firmware_base_uri, basename, NULL);
 	}
-	if (g_strstr_len(url, -1, "/") == NULL) {
+	if (g_strstr_len(url_noauth, -1, "/") == NULL) {
 		g_autofree gchar *basename = g_path_get_dirname(priv->metadata_uri);
-		return g_build_filename(basename, url, NULL);
+		return g_build_filename(basename, url_noauth, NULL);
 	}
-	return g_strdup(url);
+	return g_strdup(url_noauth);
 #endif
 }
 
-/* note, this has to be set before username and password */
-static void
+/**
+ * fwupd_remote_set_metadata_uri:
+ * @self: a #FwupdRemote
+ * @metadata_uri: (nullable): metadata URI
+ *
+ * Sets the remote metadata URI.
+ *
+ * NOTE: This has to be set before the username and password.
+ *
+ * Since: 1.8.13
+ **/
+void
 fwupd_remote_set_metadata_uri(FwupdRemote *self, const gchar *metadata_uri)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	const gchar *suffix;
 
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+
 	/* save this so we can export the object as a GVariant */
 	priv->metadata_uri = g_strdup(metadata_uri);
 
@@ -561,6 +790,13 @@ fwupd_remote_setup(FwupdRemote *self, GError **error)
 			return FALSE;
 		}
 		/* set cache to /var/lib... */
+		if (priv->metadata_uri == NULL) {
+			g_set_error_literal(error,
+					    FWUPD_ERROR,
+					    FWUPD_ERROR_INTERNAL,
+					    "metadata URI not set");
+			return FALSE;
+		}
 		if (g_str_has_suffix(priv->metadata_uri, ".xml.xz")) {
 			filename_cache =
 			    g_build_filename(priv->remotes_dir, priv->id, "metadata.xml.xz", NULL);
@@ -590,18 +826,18 @@ fwupd_remote_setup(FwupdRemote *self, GError **error)
 		}
 	}
 
-	/* load the checksum */
+	/* load the signature checksum */
 	if (priv->filename_cache_sig != NULL &&
 	    g_file_test(priv->filename_cache_sig, G_FILE_TEST_EXISTS)) {
 		gsize sz = 0;
 		g_autofree gchar *buf = NULL;
-		g_autoptr(GChecksum) checksum = g_checksum_new(G_CHECKSUM_SHA256);
+		g_autoptr(GChecksum) checksum_sig = g_checksum_new(G_CHECKSUM_SHA256);
 		if (!g_file_get_contents(priv->filename_cache_sig, &buf, &sz, error)) {
-			g_prefix_error(error, "failed to get checksum: ");
+			g_prefix_error(error, "failed to get signature checksum: ");
 			return FALSE;
 		}
-		g_checksum_update(checksum, (guchar *)buf, (gssize)sz);
-		fwupd_remote_set_checksum(self, g_checksum_get_string(checksum));
+		g_checksum_update(checksum_sig, (guchar *)buf, (gssize)sz);
+		fwupd_remote_set_checksum(self, g_checksum_get_string(checksum_sig));
 	} else {
 		fwupd_remote_set_checksum(self, NULL);
 	}
@@ -678,20 +914,30 @@ fwupd_remote_load_from_filename(FwupdRemote *self,
 		} else if (g_str_has_prefix(tmp, "http://") || g_str_has_prefix(tmp, "https://") ||
 			   g_str_has_prefix(tmp, "ipfs://") || g_str_has_prefix(tmp, "ipns://")) {
 			priv->kind = FWUPD_REMOTE_KIND_DOWNLOAD;
+			priv->refresh_interval = FWUPD_REMOTE_CONFIG_DEFAULT_REFRESH_INTERVAL;
 			fwupd_remote_set_metadata_uri(self, tmp);
 		}
 	}
 
 	/* all keys are optional */
-	if (g_key_file_has_key(kf, group, "Enabled", NULL))
-		priv->enabled = g_key_file_get_boolean(kf, group, "Enabled", NULL);
-	if (g_key_file_has_key(kf, group, "ApprovalRequired", NULL))
-		priv->approval_required =
-		    g_key_file_get_boolean(kf, group, "ApprovalRequired", NULL);
+	if (g_key_file_has_key(kf, group, "Enabled", NULL)) {
+		if (g_key_file_get_boolean(kf, group, "Enabled", NULL))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
+		else
+			fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
+	}
+	if (g_key_file_has_key(kf, group, "ApprovalRequired", NULL)) {
+		if (g_key_file_get_boolean(kf, group, "ApprovalRequired", NULL))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED);
+		else
+			fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED);
+	}
 	if (g_key_file_has_key(kf, group, "Title", NULL)) {
 		g_autofree gchar *tmp = g_key_file_get_string(kf, group, "Title", NULL);
 		fwupd_remote_set_title(self, tmp);
 	}
+	if (g_key_file_has_key(kf, group, "RefreshInterval", NULL))
+		priv->refresh_interval = g_key_file_get_uint64(kf, group, "RefreshInterval", NULL);
 	if (g_key_file_has_key(kf, group, "ReportURI", NULL)) {
 		g_autofree gchar *tmp = g_key_file_get_string(kf, group, "ReportURI", NULL);
 		fwupd_remote_set_report_uri(self, tmp);
@@ -720,18 +966,104 @@ fwupd_remote_load_from_filename(FwupdRemote *self,
 		g_autofree gchar *tmp = g_key_file_get_string(kf, group, "OrderAfter", NULL);
 		fwupd_remote_set_order_after(self, tmp);
 	}
-	if (g_key_file_has_key(kf, group, "AutomaticReports", NULL))
-		priv->automatic_reports =
-		    g_key_file_get_boolean(kf, group, "AutomaticReports", NULL);
-	if (g_key_file_has_key(kf, group, "AutomaticSecurityReports", NULL))
-		priv->automatic_security_reports =
-		    g_key_file_get_boolean(kf, group, "AutomaticSecurityReports", NULL);
+	if (g_key_file_has_key(kf, group, "AutomaticReports", NULL)) {
+		if (g_key_file_get_boolean(kf, group, "AutomaticReports", NULL))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS);
+		else
+			fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS);
+	}
+	if (g_key_file_has_key(kf, group, "AutomaticSecurityReports", NULL)) {
+		if (g_key_file_get_boolean(kf, group, "AutomaticSecurityReports", NULL))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS);
+		else
+			fwupd_remote_remove_flag(self,
+						 FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS);
+	}
+
+	/* old versions of fwupd used empty strings to mean "unset" and the package manager
+	 * might not have replaced the file marked as a config file due to modification */
+	if (g_strcmp0(priv->username, "") == 0 && g_strcmp0(priv->password, "") == 0) {
+		fwupd_remote_set_username(self, NULL);
+		fwupd_remote_set_password(self, NULL);
+	}
 
 	/* success */
 	fwupd_remote_set_filename_source(self, filename);
 	return TRUE;
 }
 
+/**
+ * fwupd_remote_save_to_filename:
+ * @self: a #FwupdRemote
+ * @filename: (not nullable): a filename
+ * @cancellable: (nullable): optional #GCancellable
+ * @error: (nullable): optional return location for an error
+ *
+ * Saves metadata about the remote to a keyfile.
+ *
+ * Returns: %TRUE for success
+ *
+ * Since: 1.8.13
+ **/
+gboolean
+fwupd_remote_save_to_filename(FwupdRemote *self,
+			      const gchar *filename,
+			      GCancellable *cancellable,
+			      GError **error)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	const gchar *group = "fwupd Remote";
+	g_autoptr(GKeyFile) kf = g_key_file_new();
+
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
+	g_return_val_if_fail(filename != NULL, FALSE);
+	g_return_val_if_fail(cancellable == NULL || G_IS_CANCELLABLE(cancellable), FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	/* optional keys */
+	if (priv->keyring_kind != FWUPD_KEYRING_KIND_UNKNOWN) {
+		g_key_file_set_string(kf,
+				      group,
+				      "Keyring",
+				      fwupd_keyring_kind_to_string(priv->keyring_kind));
+	}
+	if (priv->metadata_uri != NULL)
+		g_key_file_set_string(kf, group, "MetadataURI", priv->metadata_uri);
+	if (priv->title != NULL)
+		g_key_file_set_string(kf, group, "Title", priv->title);
+	if (priv->report_uri != NULL)
+		g_key_file_set_string(kf, group, "ReportURI", priv->report_uri);
+	if (priv->refresh_interval != 0)
+		g_key_file_set_uint64(kf, group, "RefreshInterval", priv->refresh_interval);
+	if (priv->security_report_uri != NULL)
+		g_key_file_set_string(kf, group, "SecurityReportURI", priv->security_report_uri);
+	if (priv->username != NULL)
+		g_key_file_set_string(kf, group, "Username", priv->username);
+	if (priv->password != NULL)
+		g_key_file_set_string(kf, group, "Password", priv->password);
+	if (priv->firmware_base_uri != NULL)
+		g_key_file_set_string(kf, group, "FirmwareBaseURI", priv->firmware_base_uri);
+	if (priv->order_after != NULL) {
+		g_autofree gchar *str = g_strjoinv(";", priv->order_after);
+		g_key_file_set_string(kf, group, "OrderAfter", str);
+	}
+	if (priv->order_before != NULL) {
+		g_autofree gchar *str = g_strjoinv(";", priv->order_before);
+		g_key_file_set_string(kf, group, "OrderBefore", str);
+	}
+	if (fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_ENABLED))
+		g_key_file_set_boolean(kf, group, "Enabled", TRUE);
+	if (fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED))
+		g_key_file_set_boolean(kf, group, "ApprovalRequired", TRUE);
+	if (fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS))
+		g_key_file_set_boolean(kf, group, "AutomaticReports", TRUE);
+	if (fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS))
+		g_key_file_set_boolean(kf, group, "AutomaticSecurityReports", TRUE);
+
+	/* save file */
+	return g_key_file_save_to_file(kf, filename, error);
+}
+
 /**
  * fwupd_remote_get_order_after:
  * @self: a #FwupdRemote
@@ -955,6 +1287,24 @@ fwupd_remote_set_mtime(FwupdRemote *self, guint64 mtime)
 	priv->mtime = mtime;
 }
 
+/**
+ * fwupd_remote_get_refresh_interval:
+ * @self: a #FwupdRemote
+ *
+ * Sets the plugin refresh interval in seconds.
+ *
+ * Returns: value in seconds
+ *
+ * Since: 1.9.4
+ **/
+guint64
+fwupd_remote_get_refresh_interval(FwupdRemote *self)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), G_MAXUINT64);
+	return priv->refresh_interval;
+}
+
 /**
  * fwupd_remote_get_username:
  * @self: a #FwupdRemote
@@ -1049,7 +1399,7 @@ fwupd_remote_get_remotes_dir(FwupdRemote *self)
  * fwupd_remote_get_checksum:
  * @self: a #FwupdRemote
  *
- * Gets the remote checksum.
+ * Gets the remote signature checksum.
  *
  * Returns: a string, or %NULL if unset
  *
@@ -1057,6 +1407,24 @@ fwupd_remote_get_remotes_dir(FwupdRemote *self)
  **/
 const gchar *
 fwupd_remote_get_checksum(FwupdRemote *self)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), NULL);
+	return priv->checksum_sig;
+}
+
+/**
+ * fwupd_remote_get_checksum_metadata:
+ * @self: a #FwupdRemote
+ *
+ * Gets the remote metadata checksum.
+ *
+ * Returns: a string, or %NULL if unset
+ *
+ * Since: 1.9.4
+ **/
+const gchar *
+fwupd_remote_get_checksum_metadata(FwupdRemote *self)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FWUPD_IS_REMOTE(self), NULL);
@@ -1079,10 +1447,31 @@ fwupd_remote_get_checksum(FwupdRemote *self)
 gchar *
 fwupd_remote_build_firmware_uri(FwupdRemote *self, const gchar *url, GError **error)
 {
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FWUPD_IS_REMOTE(self), NULL);
 	g_return_val_if_fail(url != NULL, NULL);
 	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
-	return fwupd_remote_build_uri(self, url, error);
+	return fwupd_remote_build_uri(self, priv->firmware_base_uri, url, error);
+}
+
+/**
+ * fwupd_remote_build_report_uri:
+ * @self: a #FwupdRemote
+ * @error: (nullable): optional return location for an error
+ *
+ * Builds a URI for the URL using the username and password set for the remote.
+ *
+ * Returns: (transfer full): a URI, or %NULL for error
+ *
+ * Since: 1.9.1
+ **/
+gchar *
+fwupd_remote_build_report_uri(FwupdRemote *self, GError **error)
+{
+	FwupdRemotePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+	return fwupd_remote_build_uri(self, NULL, priv->report_uri, error);
 }
 
 /**
@@ -1147,6 +1536,7 @@ fwupd_remote_load_signature_jcat(FwupdRemote *self, JcatFile *jcat_file, GError
 	g_autofree gchar *basename = NULL;
 	g_autofree gchar *baseuri = NULL;
 	g_autofree gchar *metadata_uri = NULL;
+	g_autoptr(GPtrArray) jcat_blobs = NULL;
 	g_autoptr(JcatItem) jcat_item = NULL;
 
 	/* this seems pointless to get the item by ID then just read the ID,
@@ -1172,11 +1562,19 @@ fwupd_remote_load_signature_jcat(FwupdRemote *self, JcatFile *jcat_file, GError
 	baseuri = g_path_get_dirname(priv->metadata_uri);
 	metadata_uri = g_build_path("/", baseuri, id, NULL);
 	if (g_strcmp0(metadata_uri, priv->metadata_uri) != 0) {
-		g_debug("changing metadata URI from %s to %s", priv->metadata_uri, metadata_uri);
+		g_info("changing metadata URI from %s to %s", priv->metadata_uri, metadata_uri);
 		g_free(priv->metadata_uri);
 		priv->metadata_uri = g_steal_pointer(&metadata_uri);
 	}
 
+	/* look for the metadata hash */
+	jcat_blobs = jcat_item_get_blobs_by_kind(jcat_item, JCAT_BLOB_KIND_SHA256);
+	if (jcat_blobs->len == 1) {
+		JcatBlob *blob = g_ptr_array_index(jcat_blobs, 0);
+		g_autofree gchar *hash = jcat_blob_get_data_as_string(blob);
+		fwupd_remote_set_checksum_metadata(self, hash);
+	}
+
 	/* success */
 	return TRUE;
 }
@@ -1300,9 +1698,27 @@ fwupd_remote_get_firmware_base_uri(FwupdRemote *self)
 gboolean
 fwupd_remote_get_enabled(FwupdRemote *self)
 {
-	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
-	return priv->enabled;
+	return fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
+}
+
+/**
+ * fwupd_remote_set_enabled:
+ * @self: a #FwupdRemote
+ * @enabled: boolean
+ *
+ * Sets if the remote is enabled and should be used.
+ *
+ * Since: 1.8.13
+ **/
+void
+fwupd_remote_set_enabled(FwupdRemote *self, gboolean enabled)
+{
+	g_return_if_fail(FWUPD_IS_REMOTE(self));
+	if (enabled)
+		fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
+	else
+		fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
 }
 
 /**
@@ -1318,9 +1734,8 @@ fwupd_remote_get_enabled(FwupdRemote *self)
 gboolean
 fwupd_remote_get_automatic_reports(FwupdRemote *self)
 {
-	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
-	return priv->automatic_reports;
+	return fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS);
 }
 
 /**
@@ -1335,10 +1750,32 @@ fwupd_remote_get_automatic_reports(FwupdRemote *self)
  **/
 gboolean
 fwupd_remote_get_automatic_security_reports(FwupdRemote *self)
+{
+	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
+	return fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS);
+}
+
+/**
+ * fwupd_remote_needs_refresh:
+ * @self: a #FwupdRemote
+ *
+ * Gets if the metadata remote needs re-downloading.
+ *
+ * Returns: a #TRUE if the remote contents are considered old
+ *
+ * Since: 1.9.4
+ **/
+gboolean
+fwupd_remote_needs_refresh(FwupdRemote *self)
 {
 	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
-	return priv->automatic_security_reports;
+
+	if (!fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_ENABLED))
+		return FALSE;
+	if (priv->kind != FWUPD_REMOTE_KIND_DOWNLOAD)
+		return FALSE;
+	return fwupd_remote_get_age(self) > priv->refresh_interval;
 }
 
 /**
@@ -1355,9 +1792,8 @@ fwupd_remote_get_automatic_security_reports(FwupdRemote *self)
 gboolean
 fwupd_remote_get_approval_required(FwupdRemote *self)
 {
-	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 	g_return_val_if_fail(FWUPD_IS_REMOTE(self), FALSE);
-	return priv->approval_required;
+	return fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED);
 }
 
 /**
@@ -1393,6 +1829,8 @@ fwupd_remote_set_from_variant_iter(FwupdRemote *self, GVariantIter *iter)
 			fwupd_remote_set_id(self, g_variant_get_string(value, NULL));
 		if (g_strcmp0(key, "Type") == 0)
 			fwupd_remote_set_kind(self, g_variant_get_uint32(value));
+		if (g_strcmp0(key, FWUPD_RESULT_KEY_FLAGS) == 0)
+			fwupd_remote_set_flags(self, g_variant_get_uint64(value));
 		if (g_strcmp0(key, "Keyring") == 0)
 			fwupd_remote_set_keyring_kind(self, g_variant_get_uint32(value));
 	}
@@ -1421,19 +1859,27 @@ fwupd_remote_set_from_variant_iter(FwupdRemote *self, GVariantIter *iter)
 		} else if (g_strcmp0(key, FWUPD_RESULT_KEY_CHECKSUM) == 0) {
 			fwupd_remote_set_checksum(self, g_variant_get_string(value, NULL));
 		} else if (g_strcmp0(key, "Enabled") == 0) {
-			priv->enabled = g_variant_get_boolean(value);
+			if (g_variant_get_boolean(value))
+				fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
 		} else if (g_strcmp0(key, "ApprovalRequired") == 0) {
-			priv->approval_required = g_variant_get_boolean(value);
+			if (g_variant_get_boolean(value))
+				fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED);
 		} else if (g_strcmp0(key, "Priority") == 0) {
 			priv->priority = g_variant_get_int32(value);
 		} else if (g_strcmp0(key, "ModificationTime") == 0) {
 			priv->mtime = g_variant_get_uint64(value);
+		} else if (g_strcmp0(key, "RefreshInterval") == 0) {
+			priv->refresh_interval = g_variant_get_uint64(value);
 		} else if (g_strcmp0(key, "FirmwareBaseUri") == 0) {
 			fwupd_remote_set_firmware_base_uri(self, g_variant_get_string(value, NULL));
 		} else if (g_strcmp0(key, "AutomaticReports") == 0) {
-			priv->automatic_reports = g_variant_get_boolean(value);
+			/* we can probably stop doing proxying flags when we next branch */
+			if (g_variant_get_boolean(value))
+				fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS);
 		} else if (g_strcmp0(key, "AutomaticSecurityReports") == 0) {
-			priv->automatic_security_reports = g_variant_get_boolean(value);
+			if (g_variant_get_boolean(value))
+				fwupd_remote_add_flag(self,
+						      FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS);
 		}
 	}
 }
@@ -1464,6 +1910,12 @@ fwupd_remote_to_variant(FwupdRemote *self)
 				      FWUPD_RESULT_KEY_REMOTE_ID,
 				      g_variant_new_string(priv->id));
 	}
+	if (priv->flags != 0) {
+		g_variant_builder_add(&builder,
+				      "{sv}",
+				      FWUPD_RESULT_KEY_FLAGS,
+				      g_variant_new_uint64(priv->flags));
+	}
 	if (priv->username != NULL) {
 		g_variant_builder_add(&builder,
 				      "{sv}",
@@ -1485,11 +1937,11 @@ fwupd_remote_to_variant(FwupdRemote *self)
 				      "Agreement",
 				      g_variant_new_string(priv->agreement));
 	}
-	if (priv->checksum != NULL) {
+	if (priv->checksum_sig != NULL) {
 		g_variant_builder_add(&builder,
 				      "{sv}",
 				      FWUPD_RESULT_KEY_CHECKSUM,
-				      g_variant_new_string(priv->checksum));
+				      g_variant_new_string(priv->checksum_sig));
 	}
 	if (priv->metadata_uri != NULL) {
 		g_variant_builder_add(&builder,
@@ -1536,6 +1988,12 @@ fwupd_remote_to_variant(FwupdRemote *self)
 				      "ModificationTime",
 				      g_variant_new_uint64(priv->mtime));
 	}
+	if (priv->refresh_interval != 0) {
+		g_variant_builder_add(&builder,
+				      "{sv}",
+				      "RefreshInterval",
+				      g_variant_new_uint64(priv->refresh_interval));
+	}
 	if (priv->filename_cache != NULL) {
 		g_variant_builder_add(&builder,
 				      "{sv}",
@@ -1554,19 +2012,30 @@ fwupd_remote_to_variant(FwupdRemote *self)
 				      "RemotesDir",
 				      g_variant_new_string(priv->remotes_dir));
 	}
-	g_variant_builder_add(&builder, "{sv}", "Enabled", g_variant_new_boolean(priv->enabled));
-	g_variant_builder_add(&builder,
-			      "{sv}",
-			      "ApprovalRequired",
-			      g_variant_new_boolean(priv->approval_required));
-	g_variant_builder_add(&builder,
-			      "{sv}",
-			      "AutomaticReports",
-			      g_variant_new_boolean(priv->automatic_reports));
-	g_variant_builder_add(&builder,
-			      "{sv}",
-			      "AutomaticSecurityReports",
-			      g_variant_new_boolean(priv->automatic_security_reports));
+	/* we can probably stop doing proxying flags when we next branch */
+	g_variant_builder_add(
+	    &builder,
+	    "{sv}",
+	    "Enabled",
+	    g_variant_new_boolean(fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_ENABLED)));
+	g_variant_builder_add(
+	    &builder,
+	    "{sv}",
+	    "ApprovalRequired",
+	    g_variant_new_boolean(
+		fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED)));
+	g_variant_builder_add(
+	    &builder,
+	    "{sv}",
+	    "AutomaticReports",
+	    g_variant_new_boolean(
+		fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS)));
+	g_variant_builder_add(
+	    &builder,
+	    "{sv}",
+	    "AutomaticSecurityReports",
+	    g_variant_new_boolean(
+		fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS)));
 	return g_variant_new("a{sv}", &builder);
 }
 
@@ -1578,19 +2047,28 @@ fwupd_remote_get_property(GObject *obj, guint prop_id, GValue *value, GParamSpec
 
 	switch (prop_id) {
 	case PROP_ENABLED:
-		g_value_set_boolean(value, priv->enabled);
+		g_value_set_boolean(value, fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_ENABLED));
 		break;
 	case PROP_APPROVAL_REQUIRED:
-		g_value_set_boolean(value, priv->approval_required);
+		g_value_set_boolean(
+		    value,
+		    fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED));
 		break;
 	case PROP_ID:
 		g_value_set_string(value, priv->id);
 		break;
 	case PROP_AUTOMATIC_REPORTS:
-		g_value_set_boolean(value, priv->automatic_reports);
+		g_value_set_boolean(
+		    value,
+		    fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS));
 		break;
 	case PROP_AUTOMATIC_SECURITY_REPORTS:
-		g_value_set_boolean(value, priv->automatic_security_reports);
+		g_value_set_boolean(
+		    value,
+		    fwupd_remote_has_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS));
+		break;
+	case PROP_FLAGS:
+		g_value_set_uint64(value, priv->flags);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
@@ -1602,23 +2080,38 @@ static void
 fwupd_remote_set_property(GObject *obj, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	FwupdRemote *self = FWUPD_REMOTE(obj);
-	FwupdRemotePrivate *priv = GET_PRIVATE(self);
 
 	switch (prop_id) {
 	case PROP_ENABLED:
-		priv->enabled = g_value_get_boolean(value);
+		if (g_value_get_boolean(value))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
+		else
+			fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_ENABLED);
 		break;
 	case PROP_APPROVAL_REQUIRED:
-		priv->approval_required = g_value_get_boolean(value);
+		if (g_value_get_boolean(value))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED);
+		else
+			fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_APPROVAL_REQUIRED);
 		break;
 	case PROP_ID:
 		fwupd_remote_set_id(self, g_value_get_string(value));
 		break;
 	case PROP_AUTOMATIC_REPORTS:
-		priv->automatic_reports = g_value_get_boolean(value);
+		if (g_value_get_boolean(value))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS);
+		else
+			fwupd_remote_remove_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_REPORTS);
 		break;
 	case PROP_AUTOMATIC_SECURITY_REPORTS:
-		priv->automatic_security_reports = g_value_get_boolean(value);
+		if (g_value_get_boolean(value))
+			fwupd_remote_add_flag(self, FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS);
+		else
+			fwupd_remote_remove_flag(self,
+						 FWUPD_REMOTE_FLAG_AUTOMATIC_SECURITY_REPORTS);
+		break;
+	case PROP_FLAGS:
+		fwupd_remote_set_flags(self, g_value_get_uint64(value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
@@ -1702,6 +2195,22 @@ fwupd_remote_class_init(FwupdRemoteClass *klass)
 				     FALSE,
 				     G_PARAM_READWRITE | G_PARAM_STATIC_NAME);
 	g_object_class_install_property(object_class, PROP_AUTOMATIC_SECURITY_REPORTS, pspec);
+
+	/**
+	 * FwupdRemote:flags:
+	 *
+	 * The remote flags.
+	 *
+	 * Since: 1.9.4
+	 */
+	pspec = g_param_spec_uint64("flags",
+				    NULL,
+				    NULL,
+				    0,
+				    G_MAXUINT64,
+				    0,
+				    G_PARAM_READWRITE | G_PARAM_STATIC_NAME);
+	g_object_class_install_property(object_class, PROP_FLAGS, pspec);
 }
 
 static void
@@ -1729,6 +2238,7 @@ fwupd_remote_finalize(GObject *obj)
 	g_free(priv->agreement);
 	g_free(priv->remotes_dir);
 	g_free(priv->checksum);
+	g_free(priv->checksum_sig);
 	g_free(priv->filename_cache);
 	g_free(priv->filename_cache_sig);
 	g_free(priv->filename_source);
