@@ -13,7 +13,7 @@
 #include "fu-bytes.h"
 #include "fu-chunk-private.h"
 #include "fu-common.h"
-#include "fu-string.h"
+#include "fu-mem.h"
 
 /**
  * FuChunk:
@@ -297,7 +297,11 @@ fu_chunk_export(FuChunk *self, FuFirmwareExportFlags flags, XbBuilderNode *bn)
 		g_autofree gchar *datastr = NULL;
 		g_autofree gchar *dataszstr = g_strdup_printf("0x%x", (guint)self->data_sz);
 		if (flags & FU_FIRMWARE_EXPORT_FLAG_ASCII_DATA) {
-			datastr = fu_strsafe((const gchar *)self->data, MIN(self->data_sz, 16));
+			datastr = fu_memstrsafe(self->data,
+						self->data_sz,
+						0x0,
+						MIN(self->data_sz, 16),
+						NULL);
 		} else {
 			datastr = g_base64_encode(self->data, self->data_sz);
 		}
@@ -322,9 +326,7 @@ fu_chunk_to_string(FuChunk *self)
 	fu_chunk_export(self, FU_FIRMWARE_EXPORT_FLAG_ASCII_DATA, bn);
 	return xb_builder_node_export(bn,
 				      XB_NODE_EXPORT_FLAG_FORMAT_MULTILINE |
-#if LIBXMLB_CHECK_VERSION(0, 2, 2)
 					  XB_NODE_EXPORT_FLAG_COLLAPSE_EMPTY |
-#endif
 					  XB_NODE_EXPORT_FLAG_FORMAT_INDENT,
 				      NULL);
 }
@@ -350,9 +352,7 @@ fu_chunk_array_to_string(GPtrArray *chunks)
 	}
 	return xb_builder_node_export(bn,
 				      XB_NODE_EXPORT_FLAG_FORMAT_MULTILINE |
-#if LIBXMLB_CHECK_VERSION(0, 2, 2)
 					  XB_NODE_EXPORT_FLAG_COLLAPSE_EMPTY |
-#endif
 					  XB_NODE_EXPORT_FLAG_FORMAT_INDENT,
 				      NULL);
 }
