@@ -13,7 +13,6 @@
 #include <sysinfoapi.h>
 
 #include "fu-common-private.h"
-#include "fu-path-private.h"
 
 GPtrArray *
 fu_common_get_block_devices(GError **error)
@@ -25,14 +24,6 @@ fu_common_get_block_devices(GError **error)
 	return NULL;
 }
 
-gboolean
-fu_path_fnmatch_impl(const gchar *pattern, const gchar *str)
-{
-	g_return_val_if_fail(pattern != NULL, FALSE);
-	g_return_val_if_fail(str != NULL, FALSE);
-	return PathMatchSpecA(str, pattern);
-}
-
 guint64
 fu_common_get_memory_size_impl(void)
 {
@@ -41,3 +32,9 @@ fu_common_get_memory_size_impl(void)
 	GlobalMemoryStatusEx(&status);
 	return (guint64)status.ullTotalPhys;
 }
+
+gchar *
+fu_common_get_kernel_cmdline_impl(GError **error)
+{
+	return g_strdup("");
+}
