@@ -5,11 +5,10 @@
  */
 #include "config.h"
 
-#include <fwupdplugin.h>
-
 #include "fu-analogix-common.h"
 #include "fu-analogix-device.h"
 #include "fu-analogix-firmware.h"
+#include "fu-analogix-struct.h"
 
 struct _FuAnalogixDevice {
 	FuUsbDevice parent_instance;
@@ -121,11 +120,11 @@ fu_analogix_device_receive(FuAnalogixDevice *self,
 
 static gboolean
 fu_analogix_device_get_update_status(FuAnalogixDevice *self,
-				     AnxUpdateStatus *status,
+				     FuAnalogixUpdateStatus *status,
 				     GError **error)
 {
 	for (guint i = 0; i < 3000; i++) {
-		guint8 status_tmp = UPDATE_STATUS_INVALID;
+		guint8 status_tmp = FU_ANALOGIX_UPDATE_STATUS_INVALID;
 		if (!fu_analogix_device_receive(self,
 						ANX_BB_RQT_GET_UPDATE_STATUS,
 						0,
@@ -134,7 +133,11 @@ fu_analogix_device_get_update_status(FuAnalogixDevice *self,
 						sizeof(status_tmp),
 						error))
 			return FALSE;
-		if ((status_tmp != UPDATE_STATUS_ERROR) && (status_tmp != UPDATE_STATUS_INVALID)) {
+		g_debug("status now: %s [0x%x]",
+			fu_analogix_update_status_to_string(status_tmp),
+			status_tmp);
+		if ((status_tmp != FU_ANALOGIX_UPDATE_STATUS_ERROR) &&
+		    (status_tmp != FU_ANALOGIX_UPDATE_STATUS_INVALID)) {
 			if (status != NULL)
 				*status = status_tmp;
 			return TRUE;
@@ -195,7 +198,6 @@ fu_analogix_device_setup(FuDevice *device, GError **error)
 static gboolean
 fu_analogix_device_find_interface(FuUsbDevice *device, GError **error)
 {
-#if G_USB_CHECK_VERSION(0, 3, 3)
 	GUsbDevice *usb_device = fu_usb_device_get_dev(device);
 	FuAnalogixDevice *self = FU_ANALOGIX_DEVICE(device);
 	g_autoptr(GPtrArray) intfs = NULL;
@@ -215,13 +217,6 @@ fu_analogix_device_find_interface(FuUsbDevice *device, GError **error)
 	}
 	g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_FOUND, "no update interface found");
 	return FALSE;
-#else
-	g_set_error_literal(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "this version of GUsb is not supported");
-	return FALSE;
-#endif
 }
 
 static gboolean
@@ -247,7 +242,7 @@ fu_analogix_device_write_chunks(FuAnalogixDevice *self,
 	fu_progress_set_id(progress, G_STRLOC);
 	fu_progress_set_steps(progress, chunks->len);
 	for (guint i = 0; i < chunks->len; i++) {
-		AnxUpdateStatus status = UPDATE_STATUS_INVALID;
+		FuAnalogixUpdateStatus status = FU_ANALOGIX_UPDATE_STATUS_INVALID;
 		FuChunk *chk = g_ptr_array_index(chunks, i);
 		if (!fu_analogix_device_send(self,
 					     ANX_BB_RQT_SEND_UPDATE_DATA,
@@ -277,7 +272,7 @@ fu_analogix_device_write_image(FuAnalogixDevice *self,
 			       FuProgress *progress,
 			       GError **error)
 {
-	AnxUpdateStatus status = UPDATE_STATUS_INVALID;
+	FuAnalogixUpdateStatus status = FU_ANALOGIX_UPDATE_STATUS_INVALID;
 	guint8 buf_init[4] = {0x0};
 	g_autoptr(GBytes) block_bytes = NULL;
 	g_autoptr(GPtrArray) chunks = NULL;
