@@ -42,6 +42,6 @@ fu_genesys_plugin_constructed(GObject *obj)
 static void
 fu_genesys_plugin_class_init(FuGenesysPluginClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	object_class->constructed = fu_genesys_plugin_constructed;
+	FuPluginClass *plugin_class = FU_PLUGIN_CLASS(klass);
+	plugin_class->constructed = fu_genesys_plugin_constructed;
 }
