@@ -6,22 +6,16 @@
 
 #include "config.h"
 
-#include <fwupdplugin.h>
-
 #include <errno.h>
 #include <fcntl.h>
 #include <glib/gstdio.h>
 #include <string.h>
 
+#include "fu-firehose-updater.h"
 #include "fu-mbim-qdu-updater.h"
 #include "fu-mm-device.h"
 #include "fu-mm-utils.h"
 #include "fu-qmi-pdc-updater.h"
-
-#if MM_CHECK_VERSION(1, 17, 2)
-#include "fu-firehose-updater.h"
-#endif
-
 #include "fu-sahara-loader.h"
 
 /* Amount of time for the modem to boot in fastboot mode. */
@@ -32,7 +26,7 @@
  * modem boots without SIM card inserted (and therefore the initialization
  * may be very slow) and also where carrier config switching is explicitly
  * required (e.g. if switching from the default (DF) to generic (GC).*/
-#define FU_MM_DEVICE_REMOVE_DELAY_REPROBE 120000 /* ms */
+#define FU_MM_DEVICE_REMOVE_DELAY_REPROBE 180000 /* ms */
 
 #define FU_MM_DEVICE_AT_RETRIES 3
 #define FU_MM_DEVICE_AT_DELAY	3000 /* ms */
@@ -81,18 +75,14 @@ struct _FuMmDevice {
 
 	/* mbim-qdu update logic */
 	gchar *port_mbim;
-#if MBIM_CHECK_VERSION(1, 25, 3)
 	FuMbimQduUpdater *mbim_qdu_updater;
-#endif /* MBIM_CHECK_VERSION(1,25,3) */
 
 	/* firehose update handling */
 	gchar *port_qcdm;
 	gchar *port_edl;
 	gchar *firehose_prog_file;
 	FuSaharaLoader *sahara_loader;
-#if MM_CHECK_VERSION(1, 17, 2)
 	FuFirehoseUpdater *firehose_updater;
-#endif
 
 	/* for sahara */
 	FuUsbDevice *usb_device;
@@ -136,46 +126,14 @@ fu_mm_device_get_update_methods(FuMmDevice *device)
 	return device->update_methods;
 }
 
-const gchar *
-fu_mm_device_get_detach_fastboot_at(FuMmDevice *device)
-{
-	g_return_val_if_fail(FU_IS_MM_DEVICE(device), NULL);
-	return device->detach_fastboot_at;
-}
-
-gint
-fu_mm_device_get_port_at_ifnum(FuMmDevice *device)
-{
-	g_return_val_if_fail(FU_IS_MM_DEVICE(device), -1);
-	return device->port_at_ifnum;
-}
-
-gint
-fu_mm_device_get_port_qmi_ifnum(FuMmDevice *device)
-{
-	g_return_val_if_fail(FU_IS_MM_DEVICE(device), -1);
-	return device->port_qmi_ifnum;
-}
-
-gint
-fu_mm_device_get_port_mbim_ifnum(FuMmDevice *device)
-{
-	g_return_val_if_fail(FU_IS_MM_DEVICE(device), -1);
-	return device->port_mbim_ifnum;
-}
-
 static gboolean
 validate_firmware_update_method(MMModemFirmwareUpdateMethod methods, GError **error)
 {
 	static const MMModemFirmwareUpdateMethod supported_combinations[] = {
 		MM_MODEM_FIRMWARE_UPDATE_METHOD_FASTBOOT,
 		MM_MODEM_FIRMWARE_UPDATE_METHOD_QMI_PDC | MM_MODEM_FIRMWARE_UPDATE_METHOD_FASTBOOT,
-#if MM_CHECK_VERSION(1, 17, 1)
 		MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU,
-#endif /* MM_CHECK_VERSION(1,17,1) */
-#if MM_CHECK_VERSION(1, 17, 2)
 		MM_MODEM_FIRMWARE_UPDATE_METHOD_FIREHOSE,
-#endif
 #if MM_CHECK_VERSION(1, 19, 1)
 		MM_MODEM_FIRMWARE_UPDATE_METHOD_FIREHOSE | MM_MODEM_FIRMWARE_UPDATE_METHOD_SAHARA,
 #endif
@@ -185,7 +143,7 @@ validate_firmware_update_method(MMModemFirmwareUpdateMethod methods, GError **er
 	methods_str = mm_modem_firmware_update_method_build_string_from_mask(methods);
 	for (guint i = 0; i < G_N_ELEMENTS(supported_combinations); i++) {
 		if (supported_combinations[i] == methods) {
-			g_debug("valid firmware update combination: %s", methods_str);
+			g_info("valid firmware update combination: %s", methods_str);
 			return TRUE;
 		}
 	}
@@ -296,7 +254,6 @@ fu_mm_device_probe_default(FuDevice *device, GError **error)
 		if (fu_device_get_protocols(device)->len == 0)
 			fu_device_add_protocol(device, "com.qualcomm.qmi_pdc");
 	}
-#if MM_CHECK_VERSION(1, 17, 1)
 	if (self->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU) {
 		for (guint i = 0; i < n_ports; i++) {
 			if (ports[i].type == MM_MODEM_PORT_TYPE_MBIM) {
@@ -306,8 +263,6 @@ fu_mm_device_probe_default(FuDevice *device, GError **error)
 		}
 		fu_device_add_protocol(device, "com.qualcomm.mbim_qdu");
 	}
-#endif /* MM_CHECK_VERSION(1,17,1) */
-#if MM_CHECK_VERSION(1, 17, 2)
 	if (self->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_FIREHOSE) {
 		for (guint i = 0; i < n_ports; i++) {
 			if (ports[i].type == MM_MODEM_PORT_TYPE_QCDM)
@@ -320,7 +275,6 @@ fu_mm_device_probe_default(FuDevice *device, GError **error)
 		}
 		fu_device_add_protocol(device, "com.qualcomm.firehose");
 	}
-#endif
 
 	mm_modem_port_info_array_free(ports, n_ports);
 
@@ -344,7 +298,6 @@ fu_mm_device_probe_default(FuDevice *device, GError **error)
 		return FALSE;
 	}
 
-#if MM_CHECK_VERSION(1, 17, 1)
 	/* a mbim port is required for mbim-qdu */
 	if ((self->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU) &&
 	    (self->port_mbim == NULL)) {
@@ -355,8 +308,6 @@ fu_mm_device_probe_default(FuDevice *device, GError **error)
 		return FALSE;
 	}
 
-#endif /* MM_CHECK_VERSION(1,17,1) */
-#if MM_CHECK_VERSION(1, 17, 2)
 	/* a qcdm or mbim port is required for firehose */
 	if ((self->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_FIREHOSE) &&
 	    (self->port_qcdm == NULL && self->port_mbim == NULL)) {
@@ -366,7 +317,6 @@ fu_mm_device_probe_default(FuDevice *device, GError **error)
 				    "failed to find QCDM port");
 		return FALSE;
 	}
-#endif
 
 	if (self->port_at != NULL) {
 		fu_mm_utils_get_port_info(self->port_at,
@@ -566,7 +516,6 @@ fu_mm_device_probe(FuDevice *device, GError **error)
 	return fu_mm_device_probe_udev(device, error);
 }
 
-#if MM_CHECK_VERSION(1, 17, 2)
 static gboolean
 fu_mm_device_io_open_qcdm(FuMmDevice *self, GError **error)
 {
@@ -596,8 +545,7 @@ fu_mm_device_qcdm_cmd(FuMmDevice *self, const guint8 *cmd, gsize cmd_len, GError
 
 	/* command */
 	qcdm_req = g_bytes_new(cmd, cmd_len);
-	if (g_getenv("FWUPD_MODEM_MANAGER_VERBOSE") != NULL)
-		fu_dump_bytes(G_LOG_DOMAIN, "writing", qcdm_req);
+	fu_dump_bytes(G_LOG_DOMAIN, "writing", qcdm_req);
 	if (!fu_io_channel_write_bytes(self->io_channel,
 				       qcdm_req,
 				       1500,
@@ -617,8 +565,7 @@ fu_mm_device_qcdm_cmd(FuMmDevice *self, const guint8 *cmd, gsize cmd_len, GError
 		g_prefix_error(error, "failed to read qcdm response: ");
 		return FALSE;
 	}
-	if (g_getenv("FWUPD_MODEM_MANAGER_VERBOSE") != NULL)
-		fu_dump_bytes(G_LOG_DOMAIN, "read", qcdm_res);
+	fu_dump_bytes(G_LOG_DOMAIN, "read", qcdm_res);
 
 	/* command == response */
 	if (g_bytes_compare(qcdm_res, qcdm_req) != 0) {
@@ -631,7 +578,6 @@ fu_mm_device_qcdm_cmd(FuMmDevice *self, const guint8 *cmd, gsize cmd_len, GError
 
 	return TRUE;
 }
-#endif /* MM_CHECK_VERSION(1,17,2) */
 
 typedef struct {
 	const gchar *cmd;
@@ -651,8 +597,7 @@ fu_mm_device_at_cmd_cb(FuDevice *device, gpointer user_data, GError **error)
 
 	/* command */
 	at_req = g_bytes_new(cmd_cr, strlen(cmd_cr));
-	if (g_getenv("FWUPD_MODEM_MANAGER_VERBOSE") != NULL)
-		fu_dump_bytes(G_LOG_DOMAIN, "writing", at_req);
+	fu_dump_bytes(G_LOG_DOMAIN, "writing", at_req);
 	if (!fu_io_channel_write_bytes(self->io_channel,
 				       at_req,
 				       1500,
@@ -664,7 +609,7 @@ fu_mm_device_at_cmd_cb(FuDevice *device, gpointer user_data, GError **error)
 
 	/* AT command has no response, return TRUE */
 	if (!helper->has_response) {
-		g_debug("No response expected for AT command: '%s', assuming succeed", helper->cmd);
+		g_debug("no response expected for AT command: '%s', assuming succeed", helper->cmd);
 		return TRUE;
 	}
 
@@ -678,8 +623,7 @@ fu_mm_device_at_cmd_cb(FuDevice *device, gpointer user_data, GError **error)
 		g_prefix_error(error, "failed to read response for %s: ", helper->cmd);
 		return FALSE;
 	}
-	if (g_getenv("FWUPD_MODEM_MANAGER_VERBOSE") != NULL)
-		fu_dump_bytes(G_LOG_DOMAIN, "read", at_res);
+	fu_dump_bytes(G_LOG_DOMAIN, "read", at_res);
 	buf = g_bytes_get_data(at_res, &bufsz);
 	if (bufsz < 6) {
 		g_set_error(error,
@@ -718,7 +662,7 @@ fu_mm_device_at_cmd_cb(FuDevice *device, gpointer user_data, GError **error)
 			if (g_strcmp0(parts[j], "") != 0 && g_strcmp0(parts[j], "OK") != 0) {
 				/* Set branch */
 				fu_device_set_branch(FU_DEVICE(self), parts[j]);
-				g_debug("Firmware branch reported as '%s'", parts[j]);
+				g_info("firmware branch reported as '%s'", parts[j]);
 				break;
 			}
 		}
@@ -1082,7 +1026,6 @@ fu_mm_device_write_firmware_qmi_pdc(FuDevice *device,
 	return TRUE;
 }
 
-#if MM_CHECK_VERSION(1, 17, 1) && MBIM_CHECK_VERSION(1, 25, 3)
 typedef struct {
 	FuDevice *device;
 	GMainLoop *mainloop;
@@ -1281,9 +1224,6 @@ fu_mm_device_write_firmware_mbim_qdu(FuDevice *device,
 	return TRUE;
 }
 
-#endif /* MM_CHECK_VERSION(1,17,1) && MBIM_CHECK_VERSION(1,25,3) */
-
-#if MM_CHECK_VERSION(1, 17, 2)
 static gboolean
 fu_mm_find_device_file(FuDevice *device, gpointer userdata, GError **error)
 {
@@ -1421,7 +1361,7 @@ fu_mm_device_firehose_write(FuMmDevice *self,
 					 error);
 }
 
-#if MM_CHECK_VERSION(1, 19, 1) && MBIM_CHECK_VERSION(1, 25, 7)
+#if MM_CHECK_VERSION(1, 19, 1)
 static gboolean
 fu_mm_device_sahara_open(FuMmDevice *self, GError **error)
 {
@@ -1437,7 +1377,7 @@ fu_mm_device_sahara_close(FuMmDevice *self, GError **error)
 	loader = g_steal_pointer(&self->sahara_loader);
 	return fu_sahara_loader_close(loader, error);
 }
-#endif // MM_CHECK_VERSION(1, 19, 1) && MBIM_CHECK_VERSION(1, 25, 7)
+#endif // MM_CHECK_VERSION(1, 19, 1)
 
 static gboolean
 fu_mm_setup_firmware_dir(FuMmDevice *self, GError **error)
@@ -1589,7 +1529,7 @@ fu_mm_device_write_firmware_firehose(FuDevice *device,
 
 		g_debug("found edl port: %s", self->port_edl);
 	}
-#if MM_CHECK_VERSION(1, 19, 1) && MBIM_CHECK_VERSION(1, 25, 7)
+#if MM_CHECK_VERSION(1, 19, 1)
 	else if ((FU_MM_DEVICE(self)->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_SAHARA) &&
 		 self->port_mbim != NULL) {
 		/* switch to emergency download (EDL) execution environment */
@@ -1607,7 +1547,7 @@ fu_mm_device_write_firmware_firehose(FuDevice *device,
 		if (!fu_sahara_loader_run(self->sahara_loader, firehose_prog, error))
 			return FALSE;
 	}
-#endif // MM_CHECK_VERSION(1, 19, 1) && MBIM_CHECK_VERSION(1, 25, 7)
+#endif // MM_CHECK_VERSION(1, 19, 1)
 	else {
 		g_set_error(error,
 			    FWUPD_ERROR,
@@ -1631,8 +1571,6 @@ fu_mm_device_write_firmware_firehose(FuDevice *device,
 	return TRUE;
 }
 
-#endif /* MM_CHECK_VERSION(1,17,2) */
-
 static gboolean
 fu_mm_device_write_firmware(FuDevice *device,
 			    FuFirmware *firmware,
@@ -1661,17 +1599,13 @@ fu_mm_device_write_firmware(FuDevice *device,
 							   &self->qmi_pdc_active_id,
 							   error);
 
-#if MM_CHECK_VERSION(1, 17, 1) && MBIM_CHECK_VERSION(1, 25, 3)
 	/* mbim qdu write operation */
 	if (self->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU)
 		return fu_mm_device_write_firmware_mbim_qdu(device, fw, progress, error);
 
-#endif /* MM_CHECK_VERSION(1,17,1) && MBIM_CHECK_VERSION(1,25,3) */
-#if MM_CHECK_VERSION(1, 17, 2)
 	/* firehose operation */
 	if (self->update_methods & MM_MODEM_FIRMWARE_UPDATE_METHOD_FIREHOSE)
 		return fu_mm_device_write_firmware_firehose(device, fw, progress, error);
-#endif
 
 	g_set_error(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "unsupported update method");
 	return FALSE;
@@ -1800,9 +1734,9 @@ fu_mm_device_setup_branch_at(FuMmDevice *self, GError **error)
 		return FALSE;
 
 	if (fu_device_get_branch(self) != NULL)
-		g_debug("using firmware branch: %s", fu_device_get_branch(self));
+		g_info("using firmware branch: %s", fu_device_get_branch(self));
 	else
-		g_debug("using firmware branch: default");
+		g_info("using firmware branch: default");
 
 	return TRUE;
 }
@@ -1900,11 +1834,11 @@ fu_mm_device_incorporate(FuDevice *device, FuDevice *donor_device)
 	FuMmDevice *donor = FU_MM_DEVICE(donor_device);
 
 	self->update_methods = fu_mm_device_get_update_methods(donor);
-	self->detach_fastboot_at = g_strdup(fu_mm_device_get_detach_fastboot_at(donor));
+	self->detach_fastboot_at = g_strdup(donor->detach_fastboot_at);
 	self->inhibition_uid = g_strdup(fu_mm_device_get_inhibition_uid(donor));
-	self->port_at_ifnum = fu_mm_device_get_port_at_ifnum(donor);
-	self->port_qmi_ifnum = fu_mm_device_get_port_qmi_ifnum(donor);
-	self->port_mbim_ifnum = fu_mm_device_get_port_mbim_ifnum(donor);
+	self->port_at_ifnum = donor->port_at_ifnum;
+	self->port_qmi_ifnum = donor->port_qmi_ifnum;
+	self->port_mbim_ifnum = donor->port_mbim_ifnum;
 	g_set_object(&self->manager, donor->manager);
 }
 
@@ -1927,6 +1861,7 @@ fu_mm_device_init(FuMmDevice *self)
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_REQUIRE_AC);
 	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_REPLUG_MATCH_GUID);
 	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_MD_SET_VERFMT);
+	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_ADD_INSTANCE_ID_REV);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_PLAIN);
 	fu_device_set_summary(FU_DEVICE(self), "Mobile broadband device");
@@ -2021,13 +1956,6 @@ fu_mm_shadow_device_new(FuMmDevice *device)
 	return shadow_device;
 }
 
-FuUsbDevice *
-fu_mm_device_get_usb_device(FuMmDevice *self)
-{
-	g_return_val_if_fail(FU_IS_MM_DEVICE(self), NULL);
-	return self->usb_device;
-}
-
 void
 fu_mm_device_set_usb_device(FuMmDevice *self, FuUsbDevice *usb_device)
 {
