@@ -7,8 +7,6 @@
 
 #include "config.h"
 
-#include <fwupdplugin.h>
-
 #include <libmm-glib.h>
 #include <string.h>
 
@@ -303,7 +301,7 @@ fu_mm_plugin_device_removed_cb(MMManager *manager, MMObject *modem, FuPlugin *pl
 	 * ModemManager, but it still exists in the system */
 	update_methods =
 	    MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU | MM_MODEM_FIRMWARE_UPDATE_METHOD_SAHARA;
-#elif MM_CHECK_VERSION(1, 17, 1)
+#else
 	update_methods = MM_MODEM_FIRMWARE_UPDATE_METHOD_MBIM_QDU;
 #endif
 
@@ -344,7 +342,7 @@ fu_mm_plugin_setup_manager(FuPlugin *plugin)
 		return;
 	}
 
-	g_debug("ModemManager %s is available", version);
+	g_info("ModemManager %s is available", version);
 
 	g_signal_connect(G_DBUS_OBJECT_MANAGER(self->manager),
 			 "object-added",
