@@ -11,8 +11,6 @@
 #include <linux/hidraw.h>
 #include <linux/input.h>
 #endif
-#include <fwupdplugin.h>
-
 #include "fu-nordic-hid-archive.h"
 #include "fu-nordic-hid-cfg-channel.h"
 
@@ -92,8 +90,12 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC(FuNordicCfgChannelDfuInfo, g_free);
 
 struct _FuNordicHidCfgChannel {
 	FuUdevDevice parent_instance;
+	gboolean dfu_support;
 	gchar *board_name;
 	gchar *bl_name;
+	gchar *generation;
+	guint16 vid;
+	guint16 pid;
 	guint8 flash_area_id;
 	guint32 flashed_image_len;
 	guint8 peer_id;
@@ -102,6 +104,9 @@ struct _FuNordicHidCfgChannel {
 
 G_DEFINE_TYPE(FuNordicHidCfgChannel, fu_nordic_hid_cfg_channel, FU_TYPE_UDEV_DEVICE)
 
+static FuNordicHidCfgChannel *
+fu_nordic_hid_cfg_channel_new(guint8 id);
+
 static void
 fu_nordic_hid_cfg_channel_module_option_free(FuNordicCfgChannelModuleOption *opt)
 {
@@ -109,6 +114,9 @@ fu_nordic_hid_cfg_channel_module_option_free(FuNordicCfgChannelModuleOption *opt
 	g_free(opt);
 }
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(FuNordicCfgChannelModuleOption,
+			      fu_nordic_hid_cfg_channel_module_option_free);
+
 static void
 fu_nordic_hid_cfg_channel_module_free(FuNordicCfgChannelModule *mod)
 {
@@ -154,8 +162,7 @@ fu_nordic_hid_cfg_channel_send(FuNordicHidCfgChannel *self,
 	FuUdevDevice *udev_device = fu_nordic_hid_cfg_channel_get_udev_device(self, error);
 	if (udev_device == NULL)
 		return FALSE;
-	if (g_getenv("FWUPD_NORDIC_HID_VERBOSE") != NULL)
-		fu_dump_raw(G_LOG_DOMAIN, "Sent", buf, bufsz);
+	fu_dump_raw(G_LOG_DOMAIN, "Sent", buf, bufsz);
 	if (!fu_udev_device_ioctl(udev_device,
 				  HIDIOCSFEATURE(bufsz),
 				  buf,
@@ -212,8 +219,7 @@ fu_nordic_hid_cfg_channel_receive(FuNordicHidCfgChannel *self,
 		return FALSE;
 	}
 
-	if (g_getenv("FWUPD_NORDIC_HID_VERBOSE") != NULL)
-		fu_dump_raw(G_LOG_DOMAIN, "Received", buf, bufsz);
+	fu_dump_raw(G_LOG_DOMAIN, "Received", buf, bufsz);
 	/*
 	 * [TODO]: Possibly add the report-id fix for Bluez versions < 5.56:
 	 * https://github.com/bluez/bluez/commit/35a2c50437cca4d26ac6537ce3a964bb509c9b62
@@ -444,6 +450,7 @@ fu_nordic_hid_cfg_channel_add_peers(FuNordicHidCfgChannel *self, GError **error)
 
 	while (cnt++ <= 0xFF) {
 		g_autoptr(FuNordicHidCfgChannel) peer = NULL;
+		g_autoptr(GError) error_peer = NULL;
 
 		if (!fu_nordic_hid_cfg_channel_cmd_send(self,
 							NULL,
@@ -460,15 +467,26 @@ fu_nordic_hid_cfg_channel_add_peers(FuNordicHidCfgChannel *self, GError **error)
 		if (res->data[8] == INVALID_PEER_ID)
 			return TRUE;
 
-		if (g_getenv("FWUPD_NORDIC_HID_VERBOSE") != NULL)
-			g_debug("detected peer: 0x%02x", res->data[8]);
+		g_debug("detected peer: 0x%02x", res->data[8]);
 
 		peer = fu_nordic_hid_cfg_channel_new(res->data[8]);
 		/* prohibit to close parent's communication descriptor */
 		fu_device_add_internal_flag(FU_DEVICE(peer),
 					    FU_DEVICE_INTERNAL_FLAG_USE_PARENT_FOR_OPEN);
-		/* probe&setup are the part of adding child */
 		fu_device_add_child(FU_DEVICE(self), FU_DEVICE(peer));
+
+		/* ensure that the general quirk content for Nordic HID devices is applied */
+		fu_device_add_instance_id(FU_DEVICE(peer), "HIDRAW\\VEN_1915");
+
+		/* remove child that does not support config channel DFU */
+		if (!fu_device_setup(FU_DEVICE(peer), &error_peer)) {
+			g_debug("failed to discover peer 0x%02x: %s",
+				res->data[8],
+				error_peer->message);
+			fu_device_remove_child(FU_DEVICE(self), FU_DEVICE(peer));
+		} else {
+			g_debug("peer 0x%02x discovered", res->data[8]);
+		}
 	}
 
 	g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_BROKEN_PIPE, "too many peers detected");
@@ -476,8 +494,9 @@ fu_nordic_hid_cfg_channel_add_peers(FuNordicHidCfgChannel *self, GError **error)
 }
 
 static gboolean
-fu_nordic_hid_cfg_channel_get_board_name(FuNordicHidCfgChannel *self, GError **error)
+fu_nordic_hid_cfg_channel_get_board_name_cb(FuDevice *device, gpointer user_data, GError **error)
 {
+	FuNordicHidCfgChannel *self = FU_NORDIC_HID_CFG_CHANNEL(device);
 	g_autoptr(FuNordicCfgChannelMsg) res = g_new0(FuNordicCfgChannelMsg, 1);
 
 	if (!fu_nordic_hid_cfg_channel_cmd_send(self,
@@ -490,7 +509,9 @@ fu_nordic_hid_cfg_channel_get_board_name(FuNordicHidCfgChannel *self, GError **e
 		return FALSE;
 	if (!fu_nordic_hid_cfg_channel_cmd_receive(self, CONFIG_STATUS_SUCCESS, res, error))
 		return FALSE;
-	self->board_name = fu_strsafe((const gchar *)res->data, res->data_len);
+	self->board_name = fu_memstrsafe(res->data, res->data_len, 0x0, res->data_len, error);
+	if (self->board_name == NULL)
+		return FALSE;
 
 	/* success */
 	return TRUE;
@@ -516,22 +537,25 @@ fu_nordic_hid_cfg_channel_get_bl_name(FuNordicHidCfgChannel *self, GError **erro
 			return FALSE;
 
 		/* check if not set via quirk */
-		if (self->bl_name != NULL &&
-		    strncmp(self->bl_name, (const char *)res->data, res->data_len) != 0) {
-			g_set_error(
-			    error,
-			    G_IO_ERROR,
-			    G_IO_ERROR_INVALID_DATA,
-			    "bootloader in quirk file is '%s' while the board is supporting '%s'",
-			    self->bl_name,
-			    g_strndup((const gchar *)res->data, res->data_len));
-			return FALSE;
+		if (self->bl_name != NULL) {
+			g_autofree gchar *tmp = g_strndup((const gchar *)res->data, res->data_len);
+
+			g_debug("Bootloader readout '%s' overrides bootloader from quirk '%s'",
+				tmp,
+				self->bl_name);
+			g_free(self->bl_name);
 		}
-		self->bl_name = fu_strsafe((const gchar *)res->data, res->data_len);
-	} else if (g_getenv("FWUPD_NORDIC_HID_VERBOSE") != NULL) {
-		g_debug("the board have no support of bootloader runtime detection");
+		self->bl_name = fu_memstrsafe(res->data, res->data_len, 0x0, res->data_len, error);
+		if (self->bl_name == NULL)
+			return FALSE;
+	} else {
+		g_debug("the board has no support of bootloader runtime detection");
 	}
 
+	/* always use the bank 0 for MCUBOOT bootloader that swaps images */
+	if (g_strcmp0(self->bl_name, "MCUBOOT") == 0)
+		self->flash_area_id = 0;
+
 	if (self->bl_name == NULL) {
 		g_set_error_literal(error,
 				    FWUPD_ERROR,
@@ -544,6 +568,70 @@ fu_nordic_hid_cfg_channel_get_bl_name(FuNordicHidCfgChannel *self, GError **erro
 	return TRUE;
 }
 
+static gboolean
+fu_nordic_hid_cfg_channel_get_devinfo(FuNordicHidCfgChannel *self, GError **error)
+{
+	guint8 event_id = 0;
+	g_autoptr(FuNordicCfgChannelMsg) res = g_new0(FuNordicCfgChannelMsg, 1);
+
+	/* query for the devinfo if the board supports it */
+	if (fu_nordic_hid_cfg_channel_get_event_id(self, "dfu", "devinfo", &event_id)) {
+		gchar *generation;
+
+		if (!fu_nordic_hid_cfg_channel_cmd_send(self,
+							"dfu",
+							"devinfo",
+							CONFIG_STATUS_FETCH,
+							NULL,
+							0,
+							error))
+			return FALSE;
+		if (!fu_nordic_hid_cfg_channel_cmd_receive(self, CONFIG_STATUS_SUCCESS, res, error))
+			return FALSE;
+
+		if (!fu_memread_uint16_safe(res->data,
+					    REPORT_SIZE,
+					    0x00,
+					    &self->vid,
+					    G_LITTLE_ENDIAN,
+					    error))
+			return FALSE;
+		if (!fu_memread_uint16_safe(res->data,
+					    REPORT_SIZE,
+					    0x02,
+					    &self->pid,
+					    G_LITTLE_ENDIAN,
+					    error))
+			return FALSE;
+		generation =
+		    fu_memstrsafe(res->data, res->data_len, 0x4, res->data_len - 0x04, error);
+		if (generation == NULL)
+			return FALSE;
+
+		/* check if not set via quirk */
+		if (self->generation != NULL) {
+			g_debug("generation readout '%s' overrides generation from quirk '%s'",
+				generation,
+				self->generation);
+			g_free(self->generation);
+		}
+		self->generation = generation;
+	} else {
+		g_debug("the board has no support of devinfo runtime detection");
+	}
+
+	if (self->generation == NULL) {
+		g_set_error_literal(error,
+				    FWUPD_ERROR,
+				    FWUPD_ERROR_INTERNAL,
+				    "the generation is not detected nor set via quirk");
+		return FALSE;
+	}
+
+	/* success */
+	return TRUE;
+}
+
 /*
  * NOTE:
  * For devices connected directly to the host,
@@ -571,7 +659,7 @@ fu_nordic_hid_cfg_channel_get_hwid(FuNordicHidCfgChannel *self, GError **error)
 		return FALSE;
 
 	/* allows to detect the single device connected via several interfaces */
-	physical_id = g_strdup_printf("%s-%02x%02x%02x%02x%02x%02x%02x%02x-%s",
+	physical_id = g_strdup_printf("%s-%02x%02x%02x%02x%02x%02x%02x%02x",
 				      self->board_name,
 				      hw_id[0],
 				      hw_id[1],
@@ -580,8 +668,7 @@ fu_nordic_hid_cfg_channel_get_hwid(FuNordicHidCfgChannel *self, GError **error)
 				      hw_id[4],
 				      hw_id[5],
 				      hw_id[6],
-				      hw_id[7],
-				      self->bl_name);
+				      hw_id[7]);
 	fu_device_set_physical_id(FU_DEVICE(self), physical_id);
 
 	/* success */
@@ -594,7 +681,7 @@ fu_nordic_hid_cfg_channel_load_module_opts(FuNordicHidCfgChannel *self,
 					   GError **error)
 {
 	for (guint8 i = 0; i < 0xFF; i++) {
-		FuNordicCfgChannelModuleOption *opt = NULL;
+		g_autoptr(FuNordicCfgChannelModuleOption) opt = NULL;
 		g_autoptr(FuNordicCfgChannelMsg) res = g_new0(FuNordicCfgChannelMsg, 1);
 
 		if (!fu_nordic_hid_cfg_channel_cmd_send_by_id(self,
@@ -611,9 +698,11 @@ fu_nordic_hid_cfg_channel_load_module_opts(FuNordicHidCfgChannel *self,
 		if (res->data[0] == END_OF_TRANSFER_CHAR)
 			break;
 		opt = g_new0(FuNordicCfgChannelModuleOption, 1);
-		opt->name = fu_strsafe((const gchar *)res->data, res->data_len);
+		opt->name = fu_memstrsafe(res->data, res->data_len, 0x0, res->data_len, error);
+		if (opt->name == NULL)
+			return FALSE;
 		opt->idx = i;
-		g_ptr_array_add(mod->options, opt);
+		g_ptr_array_add(mod->options, g_steal_pointer(&opt));
 	}
 
 	/* success */
@@ -706,9 +795,6 @@ fu_nordic_hid_cfg_channel_dfu_fwinfo(FuNordicHidCfgChannel *self, GError **error
 	}
 	/* set the target flash ID area */
 	self->flash_area_id = res->data[0] ^ 1;
-	/* always use the bank 0 for MCUBOOT bootloader */
-	if (g_strcmp0(self->bl_name, "MCUBOOT") == 0)
-		self->flash_area_id = 0;
 
 	if (!fu_memread_uint32_safe(res->data,
 				    REPORT_SIZE,
@@ -934,29 +1020,98 @@ fu_nordic_hid_cfg_channel_probe(FuDevice *device, GError **error)
 }
 
 static gboolean
-fu_nordic_hid_cfg_channel_setup(FuDevice *device, GError **error)
+fu_nordic_hid_cfg_channel_generate_ids(FuNordicHidCfgChannel *self, GError **error)
 {
-	FuNordicHidCfgChannel *self = FU_NORDIC_HID_CFG_CHANNEL(device);
+	FuDevice *device = FU_DEVICE(self);
 
-	/* get the board name */
-	if (!fu_nordic_hid_cfg_channel_get_board_name(self, error))
+	/* generate IDs */
+	fu_device_add_instance_strsafe(device, "BOARD", self->board_name);
+	fu_device_add_instance_strsafe(device, "BL", self->bl_name);
+	fu_device_add_instance_strsafe(device, "GEN", self->generation);
+
+	/* If available, use VID and PID fetched in devinfo. Otherwise, use hardcoded VID and PID of
+	 * 0x00 only for devices connected via dongle. This prevents from inheriting VID and PID of
+	 * the dongle.
+	 */
+	if ((self->vid != 0x00 && self->pid != 0x00) || (self->peer_id != 0)) {
+		fu_device_add_instance_u16(device, "VEN", self->vid);
+		fu_device_add_instance_u16(device, "DEV", self->pid);
+	}
+
+	/* For the default generation, generate GUID without the generation parameter.
+	 * Required for compatibility with already released application images.
+	 */
+	if (g_strcmp0(self->generation, "default") == 0) {
+		if (!fu_device_build_instance_id(device,
+						 error,
+						 "HIDRAW",
+						 "VEN",
+						 "DEV",
+						 "BOARD",
+						 "BL",
+						 NULL)) {
+			g_prefix_error(error, "failed to add ID without generation: ");
+			return FALSE;
+		}
+	}
+
+	if (!fu_device_build_instance_id(device,
+					 error,
+					 "HIDRAW",
+					 "VEN",
+					 "DEV",
+					 "BOARD",
+					 "BL",
+					 "GEN",
+					 NULL)) {
+		g_prefix_error(error, "failed to add complete ID: ");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+fu_nordic_hid_cfg_channel_direct_discovery(FuNordicHidCfgChannel *self, GError **error)
+{
+	g_autoptr(GError) error_board_name = NULL;
+	g_autoptr(GError) error_fwinfo = NULL;
+	FuDevice *device = FU_DEVICE(self);
+
+	/* Get the board name. The first configuration channel operation is used to check if
+	 * hidraw instance supports the protocol. In case of failure, the hidraw instance is ignored
+	 * and predefined error code is returned to suppress warning log. This is needed to properly
+	 * handle hidraw instances that do not handle configuration channel requests. A device may
+	 * not support configuration channel at all (no configuration channel HID feature report).
+	 * The configuration channel requests are handled only by the first HID instance on device
+	 * (other instances reject the configuration channel operations).
+	 *
+	 * If the HID device is connected over BLE, the configuration channel operations right after
+	 * reconnection may fail with an ioctl error. Retry after a delay to ensure that the device
+	 * will be properly recognized by the fwupd tool.
+	 */
+	if (!fu_device_retry_full(device,
+				  fu_nordic_hid_cfg_channel_get_board_name_cb,
+				  3,
+				  50,
+				  NULL,
+				  &error_board_name)) {
+		g_set_error(error,
+			    FWUPD_ERROR,
+			    FWUPD_ERROR_NOT_SUPPORTED,
+			    "Get board name failed: %s",
+			    error_board_name->message);
+		return FALSE;
+	}
+
+	/* set the physical id based on board name and HW id to detect if the device is connected
+	 * via several interfaces
+	 */
+	if (!fu_nordic_hid_cfg_channel_get_hwid(self, error))
 		return FALSE;
 	/* detect available modules first */
 	if (!fu_nordic_hid_cfg_channel_get_modinfo(self, error))
 		return FALSE;
-	/* detect bootloader type */
-	if (!fu_nordic_hid_cfg_channel_get_bl_name(self, error))
-		return FALSE;
-	/* set the physical id based on name, HW id and bootloader type of the board
-	 * to detect if the device is connected via several interfaces */
-	if (!fu_nordic_hid_cfg_channel_get_hwid(self, error))
-		return FALSE;
-	/* get device info and version */
-	if (!fu_nordic_hid_cfg_channel_dfu_fwinfo(self, error))
-		return FALSE;
-	/* check if any peer is connected via this device */
-	if (!fu_nordic_hid_cfg_channel_add_peers(self, error))
-		return FALSE;
 
 	/* generate the custom visible name for the device if absent */
 	if (fu_device_get_name(device) == NULL) {
@@ -965,17 +1120,42 @@ fu_nordic_hid_cfg_channel_setup(FuDevice *device, GError **error)
 		fu_device_set_name(device, physical_id);
 	}
 
-	/* generate IDs */
-	fu_device_add_instance_strsafe(device, "BOARD", self->board_name);
-	fu_device_add_instance_strsafe(device, "BL", self->bl_name);
-	return fu_device_build_instance_id(device,
-					   error,
-					   "HIDRAW",
-					   "VEN",
-					   "DEV",
-					   "BOARD",
-					   "BL",
-					   NULL);
+	/* get device info and version */
+	if (!fu_nordic_hid_cfg_channel_dfu_fwinfo(self, &error_fwinfo))
+		/* lack of firmware info support indicates that device does not support DFU. */
+		return TRUE;
+
+	/* detect bootloader type */
+	if (!fu_nordic_hid_cfg_channel_get_bl_name(self, error))
+		return FALSE;
+	/* detect vendor ID, product ID and generation */
+	if (!fu_nordic_hid_cfg_channel_get_devinfo(self, error))
+		return FALSE;
+
+	/* generate device IDs. */
+	if (!fu_nordic_hid_cfg_channel_generate_ids(self, error))
+		return FALSE;
+
+	self->dfu_support = TRUE;
+	fu_device_add_flag(device, FWUPD_DEVICE_FLAG_UPDATABLE);
+	fu_device_add_flag(device, FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
+
+	return TRUE;
+}
+
+static gboolean
+fu_nordic_hid_cfg_channel_setup(FuDevice *device, GError **error)
+{
+	FuNordicHidCfgChannel *self = FU_NORDIC_HID_CFG_CHANNEL(device);
+
+	if (!fu_nordic_hid_cfg_channel_direct_discovery(self, error))
+		return FALSE;
+
+	/* check if any peer is connected via this device */
+	if (!fu_nordic_hid_cfg_channel_add_peers(self, error))
+		return FALSE;
+
+	return TRUE;
 }
 
 static void
@@ -1002,11 +1182,22 @@ static void
 fu_nordic_hid_cfg_channel_to_string(FuDevice *device, guint idt, GString *str)
 {
 	FuNordicHidCfgChannel *self = FU_NORDIC_HID_CFG_CHANNEL(device);
+
 	fu_string_append(str, idt, "BoardName", self->board_name);
-	fu_string_append(str, idt, "Bootloader", self->bl_name);
-	fu_string_append_kx(str, idt, "FlashAreaId", self->flash_area_id);
-	fu_string_append_kx(str, idt, "FlashedImageLen", self->flashed_image_len);
 	fu_string_append_kx(str, idt, "PeerId", self->peer_id);
+
+	if (self->vid != 0x00 && self->pid != 0x00) {
+		fu_string_append_kx(str, idt, "VendorId", self->vid);
+		fu_string_append_kx(str, idt, "ProductId", self->pid);
+	}
+
+	if (self->dfu_support) {
+		fu_string_append(str, idt, "Bootloader", self->bl_name);
+		fu_string_append(str, idt, "Generation", self->generation);
+		fu_string_append_kx(str, idt, "FlashAreaId", self->flash_area_id);
+		fu_string_append_kx(str, idt, "FlashedImageLen", self->flashed_image_len);
+	}
+
 	for (guint i = 0; i < self->modules->len; i++) {
 		FuNordicCfgChannelModule *mod = g_ptr_array_index(self->modules, i);
 		g_autofree gchar *title = g_strdup_printf("Module%02x", i);
@@ -1169,17 +1360,26 @@ fu_nordic_hid_cfg_channel_set_quirk_kv(FuDevice *device,
 	FuNordicHidCfgChannel *self = FU_NORDIC_HID_CFG_CHANNEL(device);
 
 	if (g_strcmp0(key, "NordicHidBootloader") == 0) {
-		if (g_strcmp0(value, "B0") == 0)
-			self->bl_name = g_strdup("B0");
-		else if (g_strcmp0(value, "MCUBOOT") == 0)
-			self->bl_name = g_strdup("MCUBOOT");
-		else {
+		if (g_strcmp0(value, "B0") != 0) {
 			g_set_error_literal(error,
 					    G_IO_ERROR,
 					    G_IO_ERROR_INVALID_DATA,
-					    "must be 'B0' or 'MCUBOOT'");
+					    "can be only 'B0' in quirk");
 			return FALSE;
 		}
+		self->bl_name = g_strdup(value);
+		return TRUE;
+	}
+
+	if (g_strcmp0(key, "NordicHidGeneration") == 0) {
+		if (g_strcmp0(value, "default") != 0) {
+			g_set_error_literal(error,
+					    G_IO_ERROR,
+					    G_IO_ERROR_INVALID_DATA,
+					    "can be only 'default' in quirk");
+			return FALSE;
+		}
+		self->generation = g_strdup(value);
 		return TRUE;
 	}
 
@@ -1194,6 +1394,7 @@ fu_nordic_hid_cfg_channel_finalize(GObject *object)
 	FuNordicHidCfgChannel *self = FU_NORDIC_HID_CFG_CHANNEL(object);
 	g_free(self->board_name);
 	g_free(self->bl_name);
+	g_free(self->generation);
 	g_ptr_array_unref(self->modules);
 	G_OBJECT_CLASS(fu_nordic_hid_cfg_channel_parent_class)->finalize(object);
 }
@@ -1220,14 +1421,13 @@ fu_nordic_hid_cfg_channel_init(FuNordicHidCfgChannel *self)
 	    g_ptr_array_new_with_free_func((GDestroyNotify)fu_nordic_hid_cfg_channel_module_free);
 
 	fu_device_set_vendor(FU_DEVICE(self), "Nordic");
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_set_version_format(FU_DEVICE(self), FWUPD_VERSION_FORMAT_QUAD);
 	fu_device_add_protocol(FU_DEVICE(self), "com.nordic.hidcfgchannel");
 	fu_device_retry_set_delay(FU_DEVICE(self), FU_NORDIC_HID_CFG_CHANNEL_RETRY_DELAY);
 	fu_device_set_firmware_gtype(FU_DEVICE(self), FU_TYPE_NORDIC_HID_ARCHIVE);
 }
 
-FuNordicHidCfgChannel *
+static FuNordicHidCfgChannel *
 fu_nordic_hid_cfg_channel_new(guint8 id)
 {
 	FuNordicHidCfgChannel *self = g_object_new(FU_TYPE_NORDIC_HID_CFG_CHANNEL, NULL);
