@@ -16,6 +16,7 @@
 #include "fu-synaptics-mst-connection.h"
 #include "fu-synaptics-mst-device.h"
 #include "fu-synaptics-mst-firmware.h"
+#include "fu-synaptics-mst-struct.h"
 
 #define FU_SYNAPTICS_MST_ID_CTRL_SIZE	 0x1000
 #define SYNAPTICS_UPDATE_ENUMERATE_TRIES 3
@@ -78,14 +79,27 @@ fu_synaptics_mst_device_finalize(GObject *object)
 	G_OBJECT_CLASS(fu_synaptics_mst_device_parent_class)->finalize(object);
 }
 
+static void
+fu_synaptics_mst_device_udev_device_notify_cb(FuUdevDevice *udev_device,
+					      GParamSpec *pspec,
+					      gpointer user_data)
+{
+	FuSynapticsMstDevice *self = FU_SYNAPTICS_MST_DEVICE(user_data);
+	if (fu_udev_device_get_dev(FU_UDEV_DEVICE(self)) != NULL) {
+		fu_udev_device_set_flags(FU_UDEV_DEVICE(self),
+					 FU_UDEV_DEVICE_FLAG_OPEN_READ |
+					     FU_UDEV_DEVICE_FLAG_OPEN_WRITE |
+					     FU_UDEV_DEVICE_FLAG_VENDOR_FROM_PARENT);
+	} else {
+		fu_udev_device_set_flags(FU_UDEV_DEVICE(self),
+					 FU_UDEV_DEVICE_FLAG_OPEN_READ |
+					     FU_UDEV_DEVICE_FLAG_VENDOR_FROM_PARENT);
+	}
+}
+
 static void
 fu_synaptics_mst_device_init(FuSynapticsMstDevice *self)
 {
-	FuUdevDeviceFlags flags =
-	    FU_UDEV_DEVICE_FLAG_OPEN_READ | FU_UDEV_DEVICE_FLAG_VENDOR_FROM_PARENT;
-	if (fu_udev_device_get_dev(FU_UDEV_DEVICE(self)) != NULL)
-		flags |= FU_UDEV_DEVICE_FLAG_OPEN_WRITE;
-	fu_udev_device_set_flags(FU_UDEV_DEVICE(self), flags);
 	fu_device_add_protocol(FU_DEVICE(self), "com.synaptics.mst");
 	fu_device_set_vendor(FU_DEVICE(self), "Synaptics");
 	fu_device_add_vendor_id(FU_DEVICE(self), "DRM_DP_AUX_DEV:0x06CB");
@@ -97,6 +111,12 @@ fu_synaptics_mst_device_init(FuSynapticsMstDevice *self)
 					"ignore-board-id");
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_internal_flag(FU_DEVICE(self), FU_DEVICE_INTERNAL_FLAG_NO_PROBE_COMPLETE);
+
+	/* this is set from ->incorporate() */
+	g_signal_connect(FU_UDEV_DEVICE(self),
+			 "notify::udev-device",
+			 G_CALLBACK(fu_synaptics_mst_device_udev_device_notify_cb),
+			 self);
 }
 
 static void
@@ -344,7 +364,7 @@ fu_synaptics_mst_device_update_esm(FuSynapticsMstDevice *self,
 			}
 		}
 
-		g_debug("Waiting for flash clear to settle");
+		g_debug("waiting for flash clear to settle");
 		fu_device_sleep(FU_DEVICE(self), FLASH_SETTLE_TIME);
 
 		/* write firmware */
@@ -425,7 +445,7 @@ fu_synaptics_mst_device_update_tesla_leaf_firmware(FuSynapticsMstDevice *self,
 
 		if (!fu_synaptics_mst_device_set_flash_sector_erase(self, 0xffff, 0, error))
 			return FALSE;
-		g_debug("Waiting for flash clear to settle");
+		g_debug("waiting for flash clear to settle");
 		fu_device_sleep(FU_DEVICE(self), FLASH_SETTLE_TIME);
 
 		fu_progress_set_steps(progress, write_loops);
@@ -556,7 +576,7 @@ fu_synaptics_mst_device_update_panamera_firmware(FuSynapticsMstDevice *self,
 	/* Current max firmware size is 104K */
 	if (fw_size < payload_len)
 		fw_size = 104 * 1024;
-	g_debug("Calculated fw size as %u", fw_size);
+	g_debug("Ccalculated fw size as %u", fw_size);
 
 	/* Update firmware */
 	write_loops = fw_size / unit_sz;
@@ -582,7 +602,7 @@ fu_synaptics_mst_device_update_panamera_firmware(FuSynapticsMstDevice *self,
 								    erase_offset,
 								    error))
 			return FALSE;
-		g_debug("Waiting for flash clear to settle");
+		g_debug("waiting for flash clear to settle");
 		fu_device_sleep(FU_DEVICE(self), FLASH_SETTLE_TIME);
 
 		/* write */
@@ -889,7 +909,7 @@ fu_synaptics_mst_device_update_cayenne_firmware(FuSynapticsMstDevice *self,
 
 		if (!fu_synaptics_mst_device_set_flash_sector_erase(self, 0xffff, 0, error))
 			return FALSE;
-		g_debug("Waiting for flash clear to settle");
+		g_debug("waiting for flash clear to settle");
 		fu_device_sleep(FU_DEVICE(self), FLASH_SETTLE_TIME);
 
 		fu_progress_set_steps(progress, write_loops);
@@ -1202,9 +1222,11 @@ fu_synaptics_mst_device_read_board_id(FuSynapticsMstDevice *self,
 		offset = (gint)ADDR_MEMORY_CUSTOMER_ID;
 		break;
 	case FU_SYNAPTICS_MST_FAMILY_CAYENNE:
-	case FU_SYNAPTICS_MST_FAMILY_SPYDER:
 		offset = (gint)ADDR_MEMORY_CUSTOMER_ID_CAYENNE;
 		break;
+	case FU_SYNAPTICS_MST_FAMILY_SPYDER:
+		offset = (gint)ADDR_MEMORY_CUSTOMER_ID_SPYDER;
+		break;
 	default:
 		g_set_error(error,
 			    FWUPD_ERROR,
