@@ -8,8 +8,6 @@
 
 #include "config.h"
 
-#include <fwupdplugin.h>
-
 #include "fu-synaptics-rmi-firmware.h"
 #include "fu-synaptics-rmi-v5-device.h"
 
@@ -146,8 +144,7 @@ fu_synaptics_rmi_v5_device_secure_check(FuDevice *device,
 	g_autoptr(GByteArray) pubkey_buf = g_byte_array_new();
 	g_autoptr(GBytes) pubkey = NULL;
 
-	if (g_getenv("FWUPD_SYNAPTICS_RMI_VERBOSE") != NULL)
-		fu_dump_bytes(G_LOG_DOMAIN, "Signature", signature);
+	fu_dump_bytes(G_LOG_DOMAIN, "Signature", signature);
 
 	f34 = fu_synaptics_rmi_device_get_function(self, 0x34, error);
 	if (f34 == NULL)
@@ -211,14 +208,12 @@ fu_synaptics_rmi_v5_device_secure_check(FuDevice *device,
 		/* success */
 		break;
 	}
-	if (g_getenv("FWUPD_SYNAPTICS_RMI_VERBOSE") != NULL) {
-		fu_dump_full(G_LOG_DOMAIN,
-			     "RSA public key",
-			     pubkey_buf->data,
-			     pubkey_buf->len,
-			     16,
-			     FU_DUMP_FLAGS_NONE);
-	}
+	fu_dump_full(G_LOG_DOMAIN,
+		     "RSA public key",
+		     pubkey_buf->data,
+		     pubkey_buf->len,
+		     16,
+		     FU_DUMP_FLAGS_NONE);
 
 	/* sanity check size */
 	if (rsa_pubkey_len != pubkey_buf->len) {
