@@ -7,11 +7,10 @@
 
 #include "config.h"
 
-#include <fwupdplugin.h>
-
 #include "fu-vli-pd-device.h"
 #include "fu-vli-pd-firmware.h"
 #include "fu-vli-pd-parade-device.h"
+#include "fu-vli-struct.h"
 
 struct _FuVliPdDevice {
 	FuVliDevice parent_instance;
@@ -33,6 +32,7 @@ fu_vli_pd_device_read_regs(FuVliPdDevice *self,
 			   gsize bufsz,
 			   GError **error)
 {
+	g_autofree gchar *title = g_strdup_printf("ReadRegs@0x%x", addr);
 	if (!g_usb_device_control_transfer(fu_usb_device_get_dev(FU_USB_DEVICE(self)),
 					   G_USB_DEVICE_DIRECTION_DEVICE_TO_HOST,
 					   G_USB_DEVICE_REQUEST_TYPE_VENDOR,
@@ -49,10 +49,7 @@ fu_vli_pd_device_read_regs(FuVliPdDevice *self,
 		g_prefix_error(error, "failed to write register @0x%x: ", addr);
 		return FALSE;
 	}
-	if (g_getenv("FWUPD_VLI_USBHUB_VERBOSE") != NULL) {
-		g_autofree gchar *title = g_strdup_printf("ReadRegs@0x%x", addr);
-		fu_dump_raw(G_LOG_DOMAIN, title, buf, bufsz);
-	}
+	fu_dump_raw(G_LOG_DOMAIN, title, buf, bufsz);
 	return TRUE;
 }
 
@@ -65,10 +62,8 @@ fu_vli_pd_device_read_reg(FuVliPdDevice *self, guint16 addr, guint8 *value, GErr
 static gboolean
 fu_vli_pd_device_write_reg(FuVliPdDevice *self, guint16 addr, guint8 value, GError **error)
 {
-	if (g_getenv("FWUPD_VLI_USBHUB_VERBOSE") != NULL) {
-		g_autofree gchar *title = g_strdup_printf("WriteReg@0x%x", addr);
-		fu_dump_raw(G_LOG_DOMAIN, title, &value, sizeof(value));
-	}
+	g_autofree gchar *title = g_strdup_printf("WriteReg@0x%x", addr);
+	fu_dump_raw(G_LOG_DOMAIN, title, &value, sizeof(value));
 	if (!g_usb_device_control_transfer(fu_usb_device_get_dev(FU_USB_DEVICE(self)),
 					   G_USB_DEVICE_DIRECTION_HOST_TO_DEVICE,
 					   G_USB_DEVICE_REQUEST_TYPE_VENDOR,
@@ -423,13 +418,13 @@ fu_vli_pd_device_prepare_firmware(FuDevice *device,
 		return NULL;
 	device_kind = fu_vli_pd_firmware_get_kind(FU_VLI_PD_FIRMWARE(firmware));
 	if (fu_vli_device_get_kind(FU_VLI_DEVICE(self)) != device_kind) {
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_INVALID_FILE,
-			    "firmware incompatible, got %s, expected %s",
-			    fu_vli_common_device_kind_to_string(device_kind),
-			    fu_vli_common_device_kind_to_string(
-				fu_vli_device_get_kind(FU_VLI_DEVICE(self))));
+		g_set_error(
+		    error,
+		    FWUPD_ERROR,
+		    FWUPD_ERROR_INVALID_FILE,
+		    "firmware incompatible, got %s, expected %s",
+		    fu_vli_device_kind_to_string(device_kind),
+		    fu_vli_device_kind_to_string(fu_vli_device_get_kind(FU_VLI_DEVICE(self))));
 		return NULL;
 	}
 
